import logging
from datetime import datetime
from enum import Enum

logger = logging.getLogger(__name__)


class LinkKind(Enum):  # urls that is possible to see in iranintl
    UNKNOWN = -1
    MAIN_PAGE = 0
    LIVE_BLOG = 1
    ARTICLE = 2
    VIDEO = 3


class Link:
    def __init__(self, url: str, title: str):
        self.title = title
        self.url = url
        self.kind: LinkKind = Link.decide_link_kind(url)

    @staticmethod
    def __is_article_url(url: str):  # https://www.iranintl.com/202506267498
        splits = url.split("/")
        return len(splits) == 4 and splits[-1].isdigit()

    @staticmethod
    def decide_link_kind(url: str):
        if not url.startswith("https://www.iranintl.com"):
            logging.debug(f"unknown url: {url}")
            return LinkKind.UNKNOWN
        if url == "https://www.iranintl.com" or url == "https://www.iranintl.com/":
            return LinkKind.MAIN_PAGE
        if (
            "liveblog" in url
        ):  # https://www.iranintl.com/fa/liveblog/live-update-jun26-2025#202506278977
            return LinkKind.LIVE_BLOG
        if (
            "video" in url
        ):  # https://www.iranintl.com/fa/video/ott_56b60da21444407c870c612451cbf9c3
            return LinkKind.VIDEO
        if Link.__is_article_url(url):
            return LinkKind.ARTICLE

        logging.debug(f"unknown url: {url}")
        return LinkKind.UNKNOWN

    def to_dict(self):
        return {"title": self.title, "url": self.url, "kind": self.kind}


class Article:# https://www.iranintl.com/202506282966
    def __init__(
        self,
        header: str,
        publish_datetime: datetime,
        update_datetime: datetime,
        paragraphs: list[str],
        urls: list[Link],
    ):
        self.header = header
        self.publish_datetime = publish_datetime
        self.update_datetime = update_datetime
        self.paragraphs = paragraphs
        self.urls = urls


class LiveBlog:
    def __init__(self, header: str, articles: list[Article]): # Article here has small(2 or 3) amount of paragraphs
        self.header = header
        self.articles = articles


class Video: # https://www.iranintl.com/fa/video/ott_e2c875cdb6694e9188f96f809b45dd72
    def __init__(
        self,
        header: str,
        video_desc: str,
        publish_datetime: datetime,
        video_caption: str,
    ):
        self.header = header
        self.video_desc = video_desc
        self.publish_datetime = publish_datetime
        self.video_caption = video_caption



from bs4 import BeautifulSoup
import requests
import random
import time
import logging
from ._utils import *
from .types import *

logger = logging.getLogger(__name__)


class IranIntl:

    def __init__(self, proxies=None, timeout=10):
        self.proxies = proxies
        self.timeout = timeout
        self.mainpage_url = "https://www.iranintl.com"

    def __fetch_html(self, url, headers={'User-Agent':'Mozilla'}):
        try:
            response = requests.get(
                url, headers=headers, proxies=self.proxies, timeout=self.timeout
            )
            response.raise_for_status()
            time.sleep(random.random())
            return response.content
        except requests.RequestException as e:
            logger.error(f"request failed for {url}: {str(e)}")
            raise

    def mainpage_links(
        self
    ) -> list[Link]:  # get all links in main page
        html = self.__fetch_html(self.mainpage_url)
        soup = BeautifulSoup(html, "html.parser")
        main_tag = soup.select_one("body > main > div")

        a_tags = main_tag.find_all("a")

        links = []
        for a_tag in a_tags:
            title = a_tag.get_text(strip=True)
            url = a_tag["href"]

            if title == "":  # look for image alt
                img = a_tag.find("img")
                title = img.get("alt", "")
            
            link = Link(url, title)
            links.append(link)

        return links

    def scrape_article(self, url: str):
        # h1
        # time
        # update_time
        # p
        # a (urls)
        html = self.__fetch_html(url)
        soup = BeautifulSoup(html, "html.parser")
        article_tag = soup.find("article")
        header_tag = article_tag.find("header")
        main_tag = article_tag.find("main")

        header1_text = header_tag.find("h1").get_text(strip=True)

        time_tags = header_tag.find_all("time")
        publish_datetime = create_datetime(time_tags[0]["datetime"])
        update_datetime = create_datetime(time_tags[-1]["datetime"])

        p_tags = main_tag.find_all("p")
        paragraphs = [p.get_text("\n", strip=True) for p in p_tags]

        a_tags = article_tag.find_all("a")
        urls = [
            Link(title=a.get_text(strip=True), url=a["href"])
            for a in a_tags
            if a.get_text(strip=True) != ""
        ]

        return Article(
            header1_text, publish_datetime, update_datetime, paragraphs, urls
        )

    def scrape_liveblog(self, url: str) -> LiveBlog:
        html = self.__fetch_html(url)
        soup = BeautifulSoup(html, "html.parser")
        parent_tag = soup.select_one("body > main > div > section")

        header1_tag = parent_tag.find("h1")
        header1_text = header1_tag.get_text(strip=True)

        article_tags = parent_tag.find_all("article")

        articles: list[Article] = []

        for article_tag in article_tags:
            time_tag = article_tag.find("time")
            dt = create_datetime(time_tag["datetime"])

            ps = [p.get_text(strip=True) for p in article_tag.find_all("p")]

            urls = [
                Link(title=u.get_text(strip=True), url=u["href"])
                for u in article_tag.find_all("a")
            ]

            header3_text = article_tag.find("h3").get_text(strip=True)

            article_obj = Article(header3_text, dt, dt, ps, urls)
            articles.append(article_obj)

        lb = LiveBlog(header1_text, articles)

        return lb

    def scrape_video(self, url: str) -> Video:
        html = self.__fetch_html(url)
        soup = BeautifulSoup(html, "html.parser")
        main_tag = soup.select_one("body > main > div > section > main")

        # frame_class=main_tag.children[0]
        content_class = main_tag.find(
            "div", {"class": "video__content"}, recursive=False
        )

        header = content_class.find("h1").get_text(strip=True)
        desc = content_class.find("p").get_text(strip=True)
        date_dow = content_class.find("span", recursive=False).get_text(strip=True)

        datetime_date_dow = persian_date_to_datetime(date_dow)

        return Video(header, desc, datetime_date_dow, "")



from setuptools import setup, find_packages

setup(
    name='iranintl',
    version='0.1.1',
    packages=find_packages(exclude=['tests']), # TODO: maybe later add tests
    install_requires=[
        'requests',      
        'beautifulsoup4',
        'persiantools'
    ],
    author='Navid Poladi',
    author_email='navidpoladi99@example.com',
    description='A Python library to scrape data from Iran International news',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    url='https://github.com/navidpgg/iranintl',
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
    python_requires='>=3.6',
)


from datetime import datetime, timezone
from persiantools.jdatetime import JalaliDate


def create_datetime(dt_text: str) -> datetime:
    """Creates a datetime for datetime formats found in iranintl articles and livepages.

    2025-06-27T17:13:30.894Z"""
    # Remove trailing 'Z' and parse timezone as UTC
    dt_text = dt_text.rstrip("Z")
    try:
        # Try parsing with fractional seconds
        dt = datetime.strptime(dt_text, "%Y-%m-%dT%H:%M:%S.%f")
    except ValueError:
        # Fallback: parse without fractional seconds
        dt = datetime.strptime(dt_text, "%Y-%m-%dT%H:%M:%S")
    # Set UTC timezone
    dt = dt.replace(tzinfo=timezone.utc)
    return dt


def persian_date_to_datetime(persian_date_str: str):
    """
    Convert a Persian date string like 'جمعه ۱۴۰۴/۴/۶' to a datetime object.

    Args:
        persian_date_str (str): Persian date string in format 'DAYOFWEEK YYYY/MM/DD'

    Returns:
        datetime.datetime: Corresponding Gregorian datetime object
    """
    # Split the string by space to separate day of week and date
    parts = persian_date_str.split()
    if len(parts) != 2:
        raise ValueError("Input format must be 'DAYOFWEEK YYYY/MM/DD'")

    # Extract the date part (e.g. '۱۴۰۴/۴/۶')
    persian_date = parts[1]

    # Persian digits to English digits mapping
    persian_digits = "۰۱۲۳۴۵۶۷۸۹"
    english_digits = "0123456789"

    # Function to convert Persian numerals to English numerals
    def convert_persian_digits(persian_num_str):
        for p, e in zip(persian_digits, english_digits):
            persian_num_str = persian_num_str.replace(p, e)
        return persian_num_str

    persian_date_english = convert_persian_digits(persian_date)

    # Parse year, month, day
    year_str, month_str, day_str = persian_date_english.split("/")
    year, month, day = int(year_str), int(month_str), int(day_str)

    # Convert Jalali date to Gregorian date
    gregorian_date = JalaliDate(year, month, day).to_gregorian()

    # Return as datetime object (at midnight)
    return datetime(gregorian_date.year, gregorian_date.month, gregorian_date.day)
